'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var redux = require('redux');
var lodash = require('lodash');
var produce = require('immer');
var produce__default = _interopDefault(produce);

/**
 * 属性类型定义
 */
(function (PROPS_TYPES) {
    PROPS_TYPES["object"] = "object";
    PROPS_TYPES["objectArray"] = "objectArray";
    PROPS_TYPES["function"] = "function";
    PROPS_TYPES["number"] = "number";
    PROPS_TYPES["numberArray"] = "numberArray";
    PROPS_TYPES["string"] = "string";
    PROPS_TYPES["stringArray"] = "stringArray";
    PROPS_TYPES["enum"] = "enum";
    PROPS_TYPES["json"] = "json";
    PROPS_TYPES["boolean"] = "boolean";
})(exports.PROPS_TYPES || (exports.PROPS_TYPES = {}));
(function (NODE_PROPS_TYPES) {
    NODE_PROPS_TYPES["reactNode"] = "reactNode";
    NODE_PROPS_TYPES["functionReactNode"] = "functionReactNode";
})(exports.NODE_PROPS_TYPES || (exports.NODE_PROPS_TYPES = {}));

var ACTION_TYPES = {
    submitConfigs: `submitConfigs`,
    searchTemplate: `searchTemplate`,
    getTemplateList: `getTemplateList`,
    outputFiles: `outputFiles`,
    addTemplateInfo: `addTemplateInfo`,
    deleteTemplate: `deleteTemplate`,
    saveTemplateInfos: `saveTemplateInfos`,
    addComponent: `addComponent`,
    copyComponent: `copyComponent`,
    onLayoutSortChange: `onLayoutSortChange`,
    clearSelectedStatus: `clearSelectedStatus`,
    selectComponent: `selectComponent`,
    clearChildNodes: `clearChildNodes`,
    deleteComponent: `deleteComponent`,
    addPropsConfig: `addPropsConfig`,
    deletePropsConfig: `deletePropsConfig`,
    changeStyles: `changeStyles`,
    resizeChange: 'resizeChange',
    changeProps: `changeProps`,
    overTarget: `overTarget`,
    clearHovered: `clearHovered`,
    undo: `undo`,
    redo: `redo`,
    getDragSource: `getDragSource`,
    getDropTarget: `getDropTarget`,
    changePlatform: `changePlatform`,
    resetProps: 'resetProps',
    resetStyles: 'resetStyles',
    clearDropTarget: 'clearDropTarget',
    clearDragSource: 'clearDragSource',
};

/**
 * 根节点key
 */
const ROOT = '0';
const copyConfig = (handleInfo, selectedKey, newKey) => {
    const { componentConfigs, propsConfigSheet } = handleInfo;
    if (propsConfigSheet[selectedKey]) {
        propsConfigSheet[newKey] = propsConfigSheet[selectedKey];
    }
    const vDom = componentConfigs[selectedKey];
    const childNodes = vDom.childNodes;
    if (!childNodes) {
        componentConfigs[newKey] = vDom;
    }
    else {
        const newVDom = { ...vDom };
        componentConfigs[newKey] = newVDom;
        if (Array.isArray(childNodes)) {
            newVDom.childNodes = copyChildNodes(handleInfo, childNodes);
        }
        else {
            const newChildNodes = {};
            lodash.each(childNodes, (nodes, propName) => {
                newChildNodes[propName] = copyChildNodes(handleInfo, nodes);
            });
            newVDom.childNodes = newChildNodes;
        }
    }
};
function copyChildNodes(handleInfo, childNodes) {
    const { componentConfigs } = handleInfo;
    const newChildKeys = [];
    for (const oldKey of childNodes) {
        const newChildKey = getNewKey(componentConfigs);
        newChildKeys.push(`${newChildKey}`);
        copyConfig(handleInfo, oldKey, newChildKey);
    }
    return newChildKeys;
}
function deleteChildNodes(handleInfo, childNodes, propName) {
    if (Array.isArray(childNodes)) {
        deleteArrChild(handleInfo, childNodes);
    }
    else {
        lodash.each(childNodes, (propChildNodes, key) => {
            if (propName) {
                if (key === propName) {
                    deleteArrChild(handleInfo, propChildNodes);
                }
            }
            else {
                deleteArrChild(handleInfo, propChildNodes);
            }
        });
    }
}
function deleteArrChild(handleInfo, childNodes) {
    for (const key of childNodes) {
        const childNodesInfo = handleInfo.componentConfigs[key].childNodes;
        if (childNodesInfo) {
            deleteChildNodes(handleInfo, childNodesInfo);
        }
        delete handleInfo.componentConfigs[key];
        delete handleInfo.propsConfigSheet[key];
    }
}
/**
 * 获取路径
 * */
function getLocation(key, propName) {
    const basePath = [key, 'childNodes'];
    return propName ? [...basePath, propName] : basePath;
}
/**
 * 生成新的Key
 * @param dragVDOMAndPropsConfig
 * @param rootKey
 */
const generateNewKey = (dragVDOMAndPropsConfig, rootKey) => {
    const { vDOMCollection, propsConfigCollection } = dragVDOMAndPropsConfig;
    const keyMap = {};
    const newVDOMCollection = {};
    let newPropsConfigCollection;
    let newKey = rootKey;
    lodash.each(vDOMCollection, (vDom, key) => {
        ++newKey;
        if (key === ROOT) {
            newKey = rootKey;
        }
        keyMap[key] = `${newKey}`;
        newVDOMCollection[newKey] = vDom;
    });
    lodash.each(newVDOMCollection, (vDom) => {
        const childNodes = vDom.childNodes;
        if (!childNodes)
            return;
        if (Array.isArray(childNodes)) {
            vDom.childNodes = childNodes.map((key) => keyMap[key]);
        }
        else {
            const newChildNodes = {};
            lodash.each(childNodes, (nodes, propName) => {
                newChildNodes[propName] = nodes.map((key) => keyMap[key]);
            });
            vDom.childNodes = newChildNodes;
        }
    });
    if (propsConfigCollection) {
        newPropsConfigCollection = {};
        lodash.each(propsConfigCollection, (v, k) => {
            newPropsConfigCollection[keyMap[k]] = v;
        });
    }
    return { newVDOMCollection, newPropsConfigCollection };
};
/**
 * 获取字段在props中的位置
 * @param fieldConfigLocation
 * @returns {string}
 */
const getFieldInPropsLocation = (fieldConfigLocation) => {
    return fieldConfigLocation
        .split('.')
        .filter((location) => location !== 'childPropsConfig');
};
/**
 * 处理子节点非空
 * @param selectedInfo
 * @param componentConfigs
 * @param payload
 */
const handleRequiredHasChild = (selectedInfo, componentConfigs) => {
    const { selectedKey, propName: selectedPropName } = selectedInfo;
    const { componentName, childNodes } = componentConfigs[selectedKey];
    const { nodePropsConfig, isRequired } = getComponentConfig(componentName);
    return ((selectedPropName &&
        nodePropsConfig[selectedPropName].isRequired &&
        !lodash.get(childNodes, selectedPropName)) ||
        (isRequired && !childNodes));
};
function shallowEqual(objA, objB) {
    for (const k of Object.keys(objA)) {
        if (objA[k] !== objB[k])
            return false;
    }
    return true;
}
function deleteChildNodesKey(childNodes, deleteKey, parentPropName) {
    if (Array.isArray(childNodes)) {
        const resultChildNodes = childNodes.filter((nodeKey) => nodeKey !== deleteKey);
        return restObject(resultChildNodes);
    }
    else {
        const resultChildNodes = childNodes[parentPropName].filter((nodeKey) => nodeKey !== deleteKey);
        if (resultChildNodes.length === 0) {
            delete childNodes[parentPropName];
        }
        else {
            childNodes[parentPropName] = resultChildNodes;
        }
        return restObject(childNodes);
    }
}
function getComponentConfig(componentName) {
    const allComponentConfigs = lodash.get(LEGO_BRIDGE, [
        'config',
        'AllComponentConfigs',
    ]);
    if (!allComponentConfigs) {
        error('Component configuration information set not found！! !');
    }
    const componentConfig = allComponentConfigs[componentName];
    if (componentName && !componentConfig) {
        warn(`${componentName} configuration information not found！! !`);
    }
    return componentConfig;
}
function isContainer(componentName) {
    const containers = lodash.get(LEGO_BRIDGE, ['config', 'containers']);
    if (!containers) {
        error(`Please configure the container category of the components`);
    }
    else {
        return containers.includes(componentName);
    }
}
function error(msg) {
    console.error(msg);
    throw new Error(msg);
}
function warn(msg) {
    const warn = lodash.get(LEGO_BRIDGE, ['config', 'warn']);
    if (warn) {
        warn(msg);
    }
    else {
        console.warn(msg);
    }
    return true;
}
function getNewKey(componentConfigs) {
    const lastKey = Object.keys(componentConfigs).pop();
    return Number(lastKey) + 1;
}
function restObject(obj, field) {
    if (field) {
        delete obj[field];
    }
    if (Object.keys(obj).length === 0)
        return undefined;
    return obj;
}
function handleRules(componentConfigs, dragKey, selectedKey, propName, isForbid) {
    /**
     * 获取当前拖拽组件的父组件约束，以及属性节点配置信息
     */
    const dragComponentName = componentConfigs[dragKey].componentName;
    const dropComponentName = componentConfigs[selectedKey].componentName;
    const { fatherNodesRule } = getComponentConfig(dragComponentName);
    const { nodePropsConfig, childNodesRule } = getComponentConfig(dropComponentName);
    /**
     * 子组件约束限制，减少不必要的组件错误嵌套
     */
    const childRules = propName
        ? nodePropsConfig[propName].childNodesRule
        : childNodesRule;
    if (childRules && !childRules.includes(dragComponentName)) {
        !isForbid &&
            warn(`${propName || dropComponentName}:only allow drag and drop to add${childRules.toString()}`);
        return true;
    }
    /**
     * 父组件约束限制，减少不必要的组件错误嵌套
     */
    if (fatherNodesRule &&
        !fatherNodesRule.includes(propName ? `${dropComponentName}.${propName}` : `${dropComponentName}`)) {
        !isForbid &&
            warn(`${dragComponentName}:Only allowed as a child node or attribute node of${fatherNodesRule.toString()}`);
        return true;
    }
    return false;
}
function combineReducers(brickReducer, customReducer) {
    return (state, action) => {
        const newState = brickReducer(state, action);
        if (customReducer) {
            return customReducer(newState, action);
        }
        else {
            return newState;
        }
    };
}
function createTemplateConfigs(templateConfigs, templatePropsConfigSheet, componentConfigs, propsConfigSheet, childNodes) {
    if (Array.isArray(childNodes)) {
        for (const key of childNodes) {
            templateConfigs[key] = componentConfigs[key];
            if (propsConfigSheet[key]) {
                templatePropsConfigSheet[key] = propsConfigSheet[key];
            }
            const { childNodes } = templateConfigs[key];
            if (childNodes)
                createTemplateConfigs(templateConfigs, templatePropsConfigSheet, componentConfigs, propsConfigSheet, childNodes);
        }
    }
    else {
        lodash.each(childNodes, (childKeys) => {
            if (!lodash.isEmpty(childKeys))
                createTemplateConfigs(templateConfigs, templatePropsConfigSheet, componentConfigs, propsConfigSheet, childKeys);
        });
    }
}
function createTemplate(state) {
    const { selectedInfo, componentConfigs, propsConfigSheet } = state;
    const { selectedKey } = selectedInfo;
    const templateConfigs = { [ROOT]: componentConfigs[selectedKey] };
    const templatePropsConfigSheet = { [ROOT]: propsConfigSheet[selectedKey] };
    const { childNodes } = componentConfigs[selectedKey];
    if (!lodash.isEmpty(childNodes)) {
        createTemplateConfigs(templateConfigs, templatePropsConfigSheet, componentConfigs, propsConfigSheet, childNodes);
    }
    return { templateConfigs, templatePropsConfigSheet };
}

/**
 * 往画板或者容器组件添加组件
 * @param state
 * @returns {{componentConfigs: *}}
 */
function addComponent(state) {
    const { undo, redo, componentConfigs, dragSource, dropTarget, selectedInfo, } = state;
    /**
     * 如果没有拖拽的组件不做添加动作, 如果没有
     */
    if (!dragSource)
        return state;
    const { vDOMCollection, dragKey, parentKey, parentPropName } = dragSource;
    /**
     * 如果没有root根节点，新添加的组件添加到root
     */
    if (!componentConfigs[ROOT]) {
        undo.push({ componentConfigs });
        redo.length = 0;
        return {
            ...state,
            componentConfigs: vDOMCollection,
            dragSource: null,
            dropTarget: null,
            undo,
            redo,
        };
    }
    const { selectedKey, propName, domTreeKeys } = dropTarget || selectedInfo || {};
    /**
     * 如果有root根节点，并且即没有选中的容器组件也没有drop的目标，那么就要回退到drag目标，
     * 添加之前的页面配置
     */
    if (!selectedKey) {
        /**
         * 如果有parentKey说明是拖拽的是新添加的组件，
         * 返回原先的state状态
         */
        if (!parentKey) {
            return { ...state, ...undo.pop(), dragSource: null };
        }
        else {
            return { ...state, dragSource: null };
        }
    }
    /**
     * 当拖拽的父key与drop目标key一致说明未移动
     * 当拖拽的key包含在drop目标的domTreeKeys,说明拖拽组件是目标组件的父组件或者是自身
     */
    if (parentKey === selectedKey ||
        domTreeKeys.includes(dragKey) ||
        handleRules(componentConfigs, dragKey, selectedKey, propName)) {
        return { ...state, dragSource: null, dropTarget: null };
    }
    parentKey && undo.push({ componentConfigs });
    redo.length = 0;
    return {
        ...state,
        componentConfigs: produce.produce(componentConfigs, (oldConfigs) => {
            //添加新组件到指定容器中
            lodash.update(oldConfigs, getLocation(selectedKey, propName), (childNodes) => {
                return childNodes ? [...childNodes, dragKey] : [dragKey];
            });
            //如果有父key说明是跨组件的拖拽，原先的父容器需要删除该组件的引用
            if (parentKey) {
                lodash.update(oldConfigs, getLocation(parentKey), (childNodes) => deleteChildNodesKey(childNodes, dragKey, parentPropName));
            }
        }),
        dragSource: null,
        dropTarget: null,
        undo,
        redo,
    };
}
/**
 * 复制组件
 * @param state
 * @returns {{componentConfigs: *}}
 */
function copyComponent(state) {
    const { undo, redo, componentConfigs, selectedInfo, propsConfigSheet } = state;
    /**
     * 未选中组件不做任何操作
     */
    if (!selectedInfo) {
        warn('Please select the node you want to copy');
        return state;
    }
    if (selectedInfo.selectedKey === ROOT) {
        warn('Prohibit copying root node');
        return state;
    }
    const { selectedKey, parentPropName, parentKey } = selectedInfo;
    const handleState = { componentConfigs, propsConfigSheet };
    undo.push({ componentConfigs, propsConfigSheet });
    redo.length = 0;
    const newKey = getNewKey(componentConfigs);
    return {
        ...state,
        ...produce.produce(handleState, (oldState) => {
            lodash.update(oldState.componentConfigs, getLocation(parentKey, parentPropName), (childNodes) => [...childNodes, `${newKey}`]);
            copyConfig(oldState, selectedKey, newKey);
        }),
        undo,
        redo,
    };
}
/**
 * 当domTree中拖拽节点调整顺序时触发
 * @param state
 * @param payload
 * @returns {{componentConfigs: *}}
 */
function onLayoutSortChange(state, payload) {
    const { sortKeys, parentKey, parentPropName, dragInfo } = payload;
    const { undo, redo, componentConfigs } = state;
    undo.push({ componentConfigs });
    redo.length = 0;
    return {
        ...state,
        componentConfigs: produce.produce(componentConfigs, (oldConfigs) => {
            lodash.update(oldConfigs, getLocation(parentKey, parentPropName), () => sortKeys);
            /**
             * dragInfo有值说明为跨组件排序，需要删除拖拽组件原先父组件中的引用
             */
            if (dragInfo) {
                const { key, parentKey, parentPropName } = dragInfo;
                lodash.update(oldConfigs, getLocation(parentKey), (childNodes) => deleteChildNodesKey(childNodes, key, parentPropName));
            }
        }),
        undo,
        redo,
    };
}
/**
 * 删除组件
 * @param state
 * @returns {{propsSetting: *, componentConfigs: *, selectedInfo: *}}
 */
function deleteComponent(state) {
    const { undo, redo, componentConfigs, selectedInfo, propsConfigSheet } = state;
    /**
     * 未选中组件将不做任何操作
     */
    if (!selectedInfo) {
        warn('Please select the components you want to delete');
        return state;
    }
    const { selectedKey, parentKey, parentPropName } = selectedInfo;
    undo.push({ componentConfigs, selectedInfo, propsConfigSheet });
    const handleState = { componentConfigs, propsConfigSheet };
    redo.length = 0;
    return {
        ...state,
        ...produce.produce(handleState, (oldState) => {
            /**
             * 如果选中的是根节点说明要删除整个页面
             */
            if (selectedKey === ROOT) {
                oldState.componentConfigs = {};
                oldState.propsConfigSheet = {};
            }
            else {
                // 删除选中组件在其父组件中的引用
                lodash.update(oldState.componentConfigs, getLocation(parentKey), (childNodes) => deleteChildNodesKey(childNodes, selectedKey, parentPropName));
                const childNodes = oldState.componentConfigs[selectedKey].childNodes;
                /**
                 * 如果childNodes有值，就遍历childNodes删除其中的子节点
                 */
                if (childNodes) {
                    deleteChildNodes(oldState, childNodes);
                }
                delete oldState.componentConfigs[selectedKey];
                delete oldState.propsConfigSheet[selectedKey];
            }
        }),
        selectedInfo: null,
        undo,
        redo,
    };
}
/**
 * 清除所有子节点
 * @param state
 * @returns {{undo: *, componentConfigs, redo: *}}
 */
function clearChildNodes(state) {
    const { componentConfigs, selectedInfo, undo, redo, propsConfigSheet } = state;
    if (!selectedInfo) {
        warn('Please select the component or property you want to clear the child nodes');
        return state;
    }
    const { selectedKey, propName } = selectedInfo;
    const childNodes = lodash.get(componentConfigs, getLocation(selectedKey));
    if (!childNodes)
        return state;
    undo.push({ componentConfigs, propsConfigSheet });
    const handleState = { componentConfigs, propsConfigSheet };
    redo.length = 0;
    return {
        ...state,
        ...produce.produce(handleState, (oldState) => {
            deleteChildNodes(oldState, childNodes, propName);
            lodash.update(oldState.componentConfigs, getLocation(selectedKey), (childNodes) => {
                /**
                 * 如果 没有propName说明要清除组件的所有子节点
                 */
                if (!propName) {
                    return undefined;
                }
                else {
                    return restObject(childNodes, propName);
                }
            });
        }),
        undo,
        redo,
    };
}

/**
 * 获取拖拽组件数据
 * @param state
 * @param payload
 * @returns {{dragSource: *}}
 */
function getDragSource(state, payload) {
    // eslint-disable-next-line prefer-const
    let { componentConfigs, undo, propsConfigSheet } = state;
    // eslint-disable-next-line prefer-const
    let { vDOMCollection, dragKey, } = payload;
    const { componentName, defaultProps, propsConfigCollection, parentKey, parentPropName, } = payload;
    /**
     * componentName有值说明为新添加的组件，为其生成vDom
     */
    dragKey = dragKey || ROOT;
    if (componentName) {
        vDOMCollection = {
            [ROOT]: {
                componentName: componentName,
                props: defaultProps,
            },
        };
    }
    /**
     * 如果componentConfigs有根节点并且vDOMCollection有值，就将vDOMCollection中的
     * vDom合并到componentConfigs，为实时拖拽预览做准备
     */
    if (componentConfigs[ROOT] && vDOMCollection) {
        undo.push({ componentConfigs, propsConfigSheet });
        const newKey = getNewKey(componentConfigs);
        dragKey = `${newKey}`;
        const { newPropsConfigCollection, newVDOMCollection } = generateNewKey({
            vDOMCollection,
            propsConfigCollection,
        }, newKey);
        componentConfigs = produce__default(componentConfigs, (oldConfigs) => {
            //为虚拟dom集合生成新的key与引用，防止多次添加同一模板造成vDom顶替
            Object.assign(oldConfigs, newVDOMCollection);
        });
        propsConfigSheet = produce__default(propsConfigSheet, (oldPropsConfig) => {
            if (newPropsConfigCollection) {
                Object.assign(oldPropsConfig, newPropsConfigCollection);
            }
        });
    }
    return {
        ...state,
        dragSource: {
            vDOMCollection,
            dragKey,
            parentKey,
            parentPropName,
        },
        componentConfigs,
        propsConfigSheet,
        undo,
    };
}
/**
 * 获取放置组件的容器组件信息
 * @param state
 * @param payload
 */
function getDropTarget(state, payload) {
    /**
     * 如果location为undefined说明当前组件不是容器组件
     * 清除dropTarget信息
     */
    const { selectedInfo, dropTarget } = state;
    const { selectedKey, domTreeKeys } = payload;
    let dropTargetResult = payload;
    if (selectedInfo) {
        const { selectedKey, propName } = selectedInfo;
        if (domTreeKeys.includes(selectedKey) && !dropTarget) {
            dropTargetResult = { selectedKey, propName, domTreeKeys };
        }
        else {
            return state;
        }
    }
    return {
        ...state,
        dropTarget: dropTargetResult,
        hoverKey: selectedKey,
    };
}
function clearDropTarget(state) {
    return {
        ...state,
        dropTarget: null,
        hoverKey: null,
    };
}
function clearDragSource(state) {
    const { dragSource } = state;
    if (dragSource)
        return { ...state, dragSource: null };
    return state;
}

/**
 * 清除hover状态
 * @param state
 * @returns {{hoverKey: null}}
 */
function clearHovered(state) {
    return {
        ...state,
        hoverKey: null,
    };
}
/**
 * 目标组件
 * @param state
 * @param payload
 */
function overTarget(state, payload) {
    const { hoverKey } = payload;
    const { hoverKey: prevHoverKey } = state;
    if (hoverKey === prevHoverKey)
        return state;
    return {
        ...state,
        hoverKey,
    };
}

/**
 * 更改平台
 * @param state
 * @param payload
 */
function changePlatform(state, payload) {
    return {
        ...state,
        platformInfo: payload,
    };
}

/**
 * 添加属性配置
 * @param state
 * @param payload
 * @returns {{propsSetting: *}}
 */
function addPropsConfig(state, payload) {
    const { selectedInfo, undo, redo, propsConfigSheet, componentConfigs } = state;
    if (!selectedInfo)
        return state;
    const { newPropField, fatherFieldLocation, childPropsConfig, propType, } = payload;
    const { selectedKey } = selectedInfo;
    let isAdd = true;
    let addPropsConfig = propsConfigSheet[selectedKey] || {};
    addPropsConfig = produce__default(addPropsConfig, (oldAddPropsConfig) => {
        const msg = `${newPropField}:This property already exists`;
        const newPropFieldConfig = {
            type: propType,
            isAdd,
        };
        if (fatherFieldLocation) {
            lodash.update(oldAddPropsConfig, fatherFieldLocation, (propsContent) => {
                // 对象数组 添加一个对象时的逻辑
                if (!newPropField)
                    return childPropsConfig;
                if (!propsContent)
                    propsContent = {};
                if (propsContent[newPropField]) {
                    isAdd = false;
                    warn(msg);
                }
                else {
                    propsContent[newPropField] = newPropFieldConfig;
                }
                return propsContent;
            });
        }
        else if (oldAddPropsConfig[newPropField]) {
            isAdd = false;
            warn(msg);
        }
        else {
            oldAddPropsConfig[newPropField] = newPropFieldConfig;
        }
    });
    if (!isAdd)
        return state;
    const { propsConfig } = getComponentConfig(componentConfigs[selectedKey].componentName);
    undo.push({ selectedInfo, propsConfigSheet });
    redo.length = 0;
    return {
        ...state,
        selectedInfo: {
            ...selectedInfo,
            propsConfig: produce__default(propsConfig, (oldPropsConfig) => {
                lodash.merge(oldPropsConfig, addPropsConfig);
            }),
        },
        propsConfigSheet: produce__default(propsConfigSheet, (oldPropsConfigSheet) => {
            oldPropsConfigSheet[selectedKey] = addPropsConfig;
        }),
        undo,
        redo,
    };
}
/**
 * 删除属性配置
 * @param state
 * @param payload
 * @returns {{propsSetting: *}}
 */
function deletePropsConfig(state, payload) {
    const { selectedInfo, undo, redo, propsConfigSheet, componentConfigs } = state;
    if (!selectedInfo)
        return state;
    const { fatherFieldLocation, field } = payload;
    const { selectedKey } = selectedInfo;
    const fieldInPropsLocation = getFieldInPropsLocation(fatherFieldLocation);
    const addPropsConfig = propsConfigSheet[selectedKey];
    const newAddPropsConfig = produce__default(addPropsConfig, (oldAddPropsConfig) => {
        if (fatherFieldLocation) {
            lodash.update(oldAddPropsConfig, fatherFieldLocation, (prevPropsConfig) => {
                return restObject(prevPropsConfig, field);
            });
        }
        else {
            return restObject(oldAddPropsConfig, field);
        }
    });
    const { propsConfig } = getComponentConfig(componentConfigs[selectedKey].componentName);
    undo.push({ selectedInfo, propsConfigSheet, componentConfigs });
    redo.length = 0;
    return {
        ...state,
        componentConfigs: produce__default(componentConfigs, (oldConfig) => {
            const fieldPath = fieldInPropsLocation[0] !== '' ? fieldInPropsLocation : [];
            const paths = [selectedKey, 'props', ...fieldPath];
            if (lodash.get(oldConfig, [...paths, field]) !== undefined) {
                lodash.update(oldConfig, paths, (prevProps) => {
                    return restObject(prevProps, field);
                });
            }
        }),
        propsConfigSheet: produce__default(propsConfigSheet, (oldPropsConfigSheet) => {
            oldPropsConfigSheet[selectedKey] = newAddPropsConfig;
        }),
        selectedInfo: {
            ...selectedInfo,
            propsConfig: produce__default(propsConfig, (oldPropsConfig) => {
                lodash.merge(oldPropsConfig, newAddPropsConfig);
            }),
        },
        undo,
        redo,
    };
}
/**
 * 提交属性
 * @param state
 * @param payload
 * @returns {{propsSetting: *, componentConfigs: *}}
 */
function changeProps(state, payload) {
    const { componentConfigs, selectedInfo, undo, redo } = state;
    if (!selectedInfo)
        return state;
    const { props } = payload;
    const { selectedKey } = selectedInfo;
    undo.push({ componentConfigs });
    redo.length = 0;
    return {
        ...state,
        componentConfigs: produce__default(componentConfigs, (oldConfigs) => {
            const style = lodash.get(oldConfigs, [selectedKey, 'props', 'style']);
            if (style) {
                oldConfigs[selectedKey].props = { ...props, style };
            }
            else {
                oldConfigs[selectedKey].props = props;
            }
        }),
        undo,
        redo,
    };
}
/**
 * 重置属性
 * @param state
 */
function resetProps(state) {
    const { selectedInfo, componentConfigs, undo, redo } = state;
    if (!selectedInfo)
        return state;
    const { selectedKey, props } = selectedInfo;
    undo.push({ componentConfigs });
    redo.length = 0;
    return {
        ...state,
        componentConfigs: produce__default(componentConfigs, (oldConfigs) => {
            const style = lodash.get(oldConfigs, [selectedKey, 'props', 'style']);
            if (style) {
                oldConfigs[selectedKey].props = { ...props, style };
            }
            else {
                oldConfigs[selectedKey].props = props;
            }
        }),
        undo,
        redo,
    };
}

/**
 * 选中组件
 * @param state
 * @param payload
 * @returns {{ undo: *, propsSetting: {propsConfig, mergePropsConfig, addPropsConfig: *, props: *}, redo: *, selectedInfo: {selectedKey: *, location: *, domTreeKeys: *[], fatherLocation: *, isContainer: boolean, style: *, componentName: *, nodePropsConfig}}}
 */
function selectComponent(state, payload) {
    const { undo, redo, selectedInfo, propsConfigSheet, componentConfigs } = state;
    const { propName, domTreeKeys, key, parentKey, parentPropName } = payload;
    if (selectedInfo) {
        const { selectedKey, propName: selectedPropName } = selectedInfo;
        if ((selectedKey === key && selectedPropName == propName) ||
            handleRequiredHasChild(selectedInfo, componentConfigs))
            return state;
        if (selectedKey === key) {
            if (propName && selectedPropName !== propName) {
                domTreeKeys.push(`${key}${propName}`);
                return {
                    ...state,
                    selectedInfo: {
                        ...selectedInfo,
                        propName,
                        domTreeKeys,
                    },
                };
            }
            else {
                return {
                    ...state,
                    selectedInfo: {
                        ...selectedInfo,
                        parentKey,
                        parentPropName,
                    },
                };
            }
        }
    }
    propName && domTreeKeys.push(`${key}${propName}`);
    const { props, componentName } = componentConfigs[key];
    const { propsConfig } = getComponentConfig(componentName);
    undo.push({ selectedInfo });
    redo.length = 0;
    return {
        ...state,
        dropTarget: null,
        selectedInfo: {
            selectedKey: key,
            propName,
            domTreeKeys,
            parentKey,
            parentPropName,
            props,
            propsConfig: produce__default(propsConfig, (oldPropsConfig) => {
                lodash.merge(oldPropsConfig, propsConfigSheet[key]);
            }),
        },
        undo,
        redo,
        hoverKey: null,
    };
}
/**
 * 清除选中状态
 * @param state
 * @returns {{undo: *, propsSetting: {}, redo: *, selectedInfo: {}}}
 */
function clearSelectedStatus(state) {
    const { selectedInfo, componentConfigs, undo, redo } = state;
    if (!selectedInfo || handleRequiredHasChild(selectedInfo, componentConfigs)) {
        return state;
    }
    undo.push({ selectedInfo });
    redo.length = 0;
    return {
        ...state,
        dropTarget: null,
        selectedInfo: null,
        undo,
        redo,
    };
}

/**
 * 样式改变时调用
 * @param state
 * @param payload
 * @returns {{propsSetting: *, componentConfigs: *}|*}
 */
function changeStyles(state, payload) {
    const { undo, redo, selectedInfo, componentConfigs } = state;
    if (!selectedInfo)
        return state;
    const { style } = payload;
    undo.push({ componentConfigs });
    redo.length = 0;
    return {
        ...state,
        componentConfigs: produce__default(componentConfigs, (oldConfigs) => {
            const config = oldConfigs[selectedInfo.selectedKey];
            if (config.props) {
                config.props.style = style;
            }
            else {
                config.props = { style };
            }
        }),
        undo,
        redo,
    };
}
function resetStyles(state) {
    const { selectedInfo, undo, componentConfigs, redo } = state;
    if (!selectedInfo)
        return state;
    const { props, selectedKey } = selectedInfo;
    undo.push({ componentConfigs });
    redo.length = 0;
    return {
        ...state,
        componentConfigs: produce__default(componentConfigs, (oldConfigs) => {
            const config = oldConfigs[selectedKey];
            if (props && props.style) {
                config.props.style = props.style;
            }
            else {
                config.props = restObject(config.props, 'style');
            }
        }),
        undo,
        redo,
    };
}
function resizeChange(state, payload) {
    const { componentConfigs, undo, redo, selectedInfo } = state;
    if (!selectedInfo)
        return state;
    const { selectedKey } = selectedInfo;
    const { width, height } = payload;
    if (width || height) {
        undo.push({ componentConfigs });
        redo.length = 0;
    }
    return {
        ...state,
        componentConfigs: produce__default(componentConfigs, (oldConfigs) => {
            if (width) {
                lodash.update(oldConfigs[selectedKey], 'props.style.width', () => width);
            }
            if (height) {
                lodash.update(oldConfigs[selectedKey], 'props.style.height', () => height);
            }
        }),
    };
}

/**
 * 撤销
 * @param state
 * @returns {(*&void&{undo: *, redo: *})|(*&T&{undo: *, redo: *})}
 */
function undo(state) {
    const { undo, redo } = state;
    if (undo.length == 0)
        return state;
    const nextState = undo.pop();
    const prevState = {};
    lodash.each(nextState, (_, key) => (prevState[key] = lodash.get(state, key)));
    redo.push(prevState);
    return {
        ...state,
        ...nextState,
        undo,
        redo,
    };
}
/**
 * 重做
 * @param state
 * @returns {(*&void&{undo: *, redo: *})|(*&T&{undo: *, redo: *})}
 */
function redo(state) {
    const { undo, redo } = state;
    if (redo.length == 0)
        return state;
    const nextState = redo.pop();
    const prevState = {};
    lodash.each(nextState, (_, key) => (prevState[key] = lodash.get(state, key)));
    undo.push(prevState);
    return {
        ...state,
        ...nextState,
        undo,
        redo,
    };
}

const reducer = (prevState, action) => {
    const state = prevState;
    const { type, payload } = action;
    switch (type) {
        case ACTION_TYPES.addComponent:
            return addComponent(state);
        case ACTION_TYPES.clearChildNodes:
            return clearChildNodes(state);
        case ACTION_TYPES.onLayoutSortChange:
            return onLayoutSortChange(state, payload);
        case ACTION_TYPES.deleteComponent:
            return deleteComponent(state);
        case ACTION_TYPES.copyComponent:
            return copyComponent(state);
        case ACTION_TYPES.getDragSource:
            return getDragSource(state, payload);
        case ACTION_TYPES.getDropTarget:
            return getDropTarget(state, payload);
        case ACTION_TYPES.clearHovered:
            return clearHovered(state);
        case ACTION_TYPES.overTarget:
            return overTarget(state, payload);
        case ACTION_TYPES.changePlatform:
            return changePlatform(state, payload);
        case ACTION_TYPES.addPropsConfig:
            return addPropsConfig(state, payload);
        case ACTION_TYPES.deletePropsConfig:
            return deletePropsConfig(state, payload);
        case ACTION_TYPES.changeProps:
            return changeProps(state, payload);
        case ACTION_TYPES.selectComponent:
            return selectComponent(state, payload);
        case ACTION_TYPES.clearSelectedStatus:
            return clearSelectedStatus(state);
        case ACTION_TYPES.changeStyles:
            return changeStyles(state, payload);
        case ACTION_TYPES.undo:
            return undo(state);
        case ACTION_TYPES.redo:
            return redo(state);
        case ACTION_TYPES.resetProps:
            return resetProps(state);
        case ACTION_TYPES.resetStyles:
            return resetStyles(state);
        case ACTION_TYPES.clearDropTarget:
            return clearDropTarget(state);
        case ACTION_TYPES.resizeChange:
            return resizeChange(state, payload);
        case ACTION_TYPES.clearDragSource:
            return clearDragSource(state);
        default:
            return state;
    }
};

const LEGO_BRIDGE = {
    store: null,
};
const legoState = {
    componentConfigs: {},
    selectedInfo: null,
    propsConfigSheet: {},
    undo: [],
    redo: [],
    hoverKey: null,
    dragSource: null,
    dropTarget: null,
    platformInfo: { isMobile: false, size: ['100%', '100%'] },
};
function createActions(action) {
    return LEGO_BRIDGE.store.dispatch(action);
}
function createLegStore(initState = {}, config, customReducer) {
    if (LEGO_BRIDGE.store)
        return LEGO_BRIDGE.store;
    if (!LEGO_BRIDGE.config && !config) {
        throw Error('config未初始化');
    }
    else if (!LEGO_BRIDGE.config) {
        LEGO_BRIDGE.config = config;
    }
    const store = redux.createStore(combineReducers(reducer, customReducer), { ...legoState, ...initState }, window.__REDUX_DEVTOOLS_EXTENSION__ &&
        window.__REDUX_DEVTOOLS_EXTENSION__());
    LEGO_BRIDGE.store = store;
    return store;
}

const addComponent$1 = () => createActions({ type: ACTION_TYPES.addComponent });
const copyComponent$1 = () => createActions({ type: ACTION_TYPES.copyComponent });
const onLayoutSortChange$1 = (payload) => createActions({
    type: ACTION_TYPES.onLayoutSortChange,
    payload,
});
const deleteComponent$1 = () => createActions({ type: ACTION_TYPES.deleteComponent });
const clearChildNodes$1 = () => createActions({ type: ACTION_TYPES.clearChildNodes });

const getDragSource$1 = (payload) => createActions({
    type: ACTION_TYPES.getDragSource,
    payload,
});
const getDropTarget$1 = (payload) => createActions({
    type: ACTION_TYPES.getDropTarget,
    payload,
});
const clearDropTarget$1 = () => createActions({
    type: ACTION_TYPES.clearDropTarget,
});
const clearDragSource$1 = () => createActions({
    type: ACTION_TYPES.clearDragSource,
});

const clearHovered$1 = () => createActions({ type: ACTION_TYPES.clearHovered });
const overTarget$1 = (payload) => createActions({
    type: ACTION_TYPES.overTarget,
    payload,
});

const changePlatform$1 = (payload) => createActions({
    type: ACTION_TYPES.changePlatform,
    payload,
});

const addPropsConfig$1 = (payload) => createActions({
    type: ACTION_TYPES.addPropsConfig,
    payload,
});
const deletePropsConfig$1 = (payload) => createActions({
    type: ACTION_TYPES.deletePropsConfig,
    payload,
});
const changeProps$1 = (payload) => createActions({
    type: ACTION_TYPES.changeProps,
    payload,
});
const resetProps$1 = () => createActions({ type: ACTION_TYPES.resetProps });

const undo$1 = () => createActions({ type: ACTION_TYPES.undo });
const redo$1 = () => createActions({ type: ACTION_TYPES.redo });

const selectComponent$1 = (payload) => createActions({
    type: ACTION_TYPES.selectComponent,
    payload,
});
const clearSelectedStatus$1 = () => createActions({ type: ACTION_TYPES.clearSelectedStatus });

const changeStyles$1 = (payload) => createActions({
    type: ACTION_TYPES.changeStyles,
    payload,
});
const resetStyles$1 = () => createActions({ type: ACTION_TYPES.resetStyles });
const resizeChange$1 = (payload) => createActions({ type: ACTION_TYPES.resizeChange, payload });

Object.defineProperty(exports, 'original', {
	enumerable: true,
	get: function () {
		return produce.original;
	}
});
Object.defineProperty(exports, 'produce', {
	enumerable: true,
	get: function () {
		return produce.produce;
	}
});
exports.LEGO_BRIDGE = LEGO_BRIDGE;
exports.ROOT = ROOT;
exports.addComponent = addComponent$1;
exports.addPropsConfig = addPropsConfig$1;
exports.changePlatform = changePlatform$1;
exports.changeProps = changeProps$1;
exports.changeStyles = changeStyles$1;
exports.clearChildNodes = clearChildNodes$1;
exports.clearDragSource = clearDragSource$1;
exports.clearDropTarget = clearDropTarget$1;
exports.clearHovered = clearHovered$1;
exports.clearSelectedStatus = clearSelectedStatus$1;
exports.combineReducers = combineReducers;
exports.copyComponent = copyComponent$1;
exports.copyConfig = copyConfig;
exports.createActions = createActions;
exports.createLegStore = createLegStore;
exports.createTemplate = createTemplate;
exports.createTemplateConfigs = createTemplateConfigs;
exports.deleteChildNodes = deleteChildNodes;
exports.deleteChildNodesKey = deleteChildNodesKey;
exports.deleteComponent = deleteComponent$1;
exports.deletePropsConfig = deletePropsConfig$1;
exports.error = error;
exports.generateNewKey = generateNewKey;
exports.getComponentConfig = getComponentConfig;
exports.getDragSource = getDragSource$1;
exports.getDropTarget = getDropTarget$1;
exports.getFieldInPropsLocation = getFieldInPropsLocation;
exports.getLocation = getLocation;
exports.getNewKey = getNewKey;
exports.handleRequiredHasChild = handleRequiredHasChild;
exports.handleRules = handleRules;
exports.isContainer = isContainer;
exports.legoState = legoState;
exports.onLayoutSortChange = onLayoutSortChange$1;
exports.overTarget = overTarget$1;
exports.redo = redo$1;
exports.resetProps = resetProps$1;
exports.resetStyles = resetStyles$1;
exports.resizeChange = resizeChange$1;
exports.restObject = restObject;
exports.selectComponent = selectComponent$1;
exports.shallowEqual = shallowEqual;
exports.undo = undo$1;
exports.warn = warn;
